@@ -1,17 +1,64 @@
 import re
 from django.conf import settings
-from django.contrib.auth.models import User
+from django.utils.safestring import mark_safe
 from django.core.paginator import Paginator, EmptyPage
 
- 
-def get_redir_path():        
-    return getattr(
+try:
+    # Django 1.5 check
+    from django.contrib.auth import get_user_model
+except ImportError:
+    from django.contrib.auth.models import User
+else:
+    User = get_user_model()
+
+
+def get_redir_path(request=None):
+    nextval = None
+    redirect_field_name = getattr(
+        settings,
+        'IMPERSONATE_REDIRECT_FIELD_NAME',
+        None,
+    )
+    if request and redirect_field_name:
+        nextval = request.GET.get(redirect_field_name, None)
+    return nextval or getattr(
         settings,
         'IMPERSONATE_REDIRECT_URL',
         getattr(settings, 'LOGIN_REDIRECT_URL', '/'),
-    )       
-        
-    
+    )
+
+
+def get_redir_arg(request):
+    redirect_field_name = getattr(
+        settings,
+        'IMPERSONATE_REDIRECT_FIELD_NAME',
+        None,
+    )
+    if redirect_field_name:
+        nextval = request.GET.get(redirect_field_name, None)
+        if nextval:
+            return '?{0}={1}'.format(redirect_field_name, nextval)
+    return ''
+
+
+def get_redir_field(request):
+    redirect_field_name = getattr(
+        settings,
+        'IMPERSONATE_REDIRECT_FIELD_NAME',
+        None,
+    )
+    if redirect_field_name:
+        nextval = request.GET.get(redirect_field_name, None)
+        if nextval:
+            return mark_safe(
+                '<input type="hidden" name="{0}" value="{1}"/>'.format(
+                    redirect_field_name,
+                    nextval,
+                )
+            )
+    return ''
+
+
 def get_paginator(request, qs):
     try:
         page_number = int(request.GET.get('page', 1))
@@ -21,7 +68,7 @@ def get_paginator(request, qs):
     paginator = Paginator(
         qs,
         int(getattr(settings, 'IMPERSONATE_PAGINATE_COUNT', 20)),
-    )   
+    )
     try:
         page = paginator.page(page_number)
     except EmptyPage:
@@ -35,8 +82,8 @@ def check_allow_staff():
 
 
 def users_impersonable(request):
-    ''' Returns a QuerySet of users that this user can impersonate. 
-        Uses the IMPERSONATE_CUSTOM_USER_QUERYSET if set, else, it 
+    ''' Returns a QuerySet of users that this user can impersonate.
+        Uses the IMPERSONATE_CUSTOM_USER_QUERYSET if set, else, it
         returns all users
     '''
     if hasattr(settings, 'IMPERSONATE_CUSTOM_USER_QUERYSET'):
@@ -53,12 +100,12 @@ def check_allow_for_user(request, end_user):
     '''
     if check_allow_impersonate(request):
         # start user can impersonate
-        # Can impersonate anyone who's not a superuser and who is in your 
+        # Can impersonate anyone who's not a superuser and who is in your
         # queryset of 'who i can impersonate'
         upk = end_user.pk
         return (
-            not end_user.is_superuser and \
-            users_impersonable(request).filter(pk=upk).count()
+            not end_user.is_superuser and
+            users_impersonable(request).filter(pk=upk).exists()
         )
 
     # start user not allowed impersonate at all
@@ -66,7 +113,7 @@ def check_allow_for_user(request, end_user):
 
 
 def import_func_from_string(string_name):
-    ''' Given a string like 'mod.mod2.funcname' which refers to a function, 
+    ''' Given a string like 'mod.mod2.funcname' which refers to a function,
         return that function so it can be called
     '''
     mod_name, func_name = string_name.rsplit('.', 1)
@@ -81,7 +128,7 @@ def import_func_from_string(string_name):
 
 def check_allow_impersonate(request):
     ''' Returns True if this request is allowed to do any impersonation.
-        Uses the IMPERSONATE_CUSTOM_ALLOW function if required, else 
+        Uses the IMPERSONATE_CUSTOM_ALLOW function if required, else
         looks at superuser/staff status and IMPERSONATE_REQUIRE_SUPERUSER
     '''
     if hasattr(settings, 'IMPERSONATE_CUSTOM_ALLOW'):
