@@ -1,5 +1,4 @@
-from django.conf import settings
-from helpers import check_allow_for_user, check_allow_for_uri
+from .helpers import check_allow_for_user, check_allow_for_uri
 
 
 class ImpersonateMiddleware(object):
